@@ -1,9 +1,9 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 '''
 A library to access Tryton's models like a client.
 '''
-__version__ = "3.4.0"
+__version__ = "3.6.1"
 __all__ = ['Model', 'Wizard', 'Report']
 import sys
 try:
@@ -16,6 +16,7 @@ except ImportError:
     sys.modules['cdecimal'] = decimal
 import threading
 import datetime
+import functools
 from decimal import Decimal
 from types import NoneType
 
@@ -73,6 +74,37 @@ class _EvalEnvironment(dict):
             return item in self.parent._fields
 
 
+class dualmethod(object):
+    """Descriptor implementing combination of class and instance method
+
+    When called on an instance, the class is passed as the first argument and a
+    list with the instance as the second.
+    When called on a class, the class itself is passed as the first argument.
+
+    >>> class Example(object):
+    ...     @dualmethod
+    ...     def method(cls, instances):
+    ...         print len(instances)
+    ...
+    >>> Example.method([Example()])
+    1
+    >>> Example().method()
+    1
+    """
+    def __init__(self, func):
+        self.func = func
+
+    def __get__(self, instance, owner):
+
+        @functools.wraps(self.func)
+        def newfunc(*args, **kwargs):
+            if instance:
+                return self.func(owner, [instance], *args, **kwargs)
+            else:
+                return self.func(owner, *args, **kwargs)
+        return newfunc
+
+
 class FieldDescriptor(object):
     default = None
 
@@ -94,10 +126,10 @@ class FieldDescriptor(object):
         instance._values[self.name] = value
         if previous != getattr(instance, self.name):
             instance._changed.add(self.name)
-            instance._on_change(self.name)
+            instance._on_change([self.name])
             if instance._parent:
                 instance._parent._changed.add(instance._parent_field_name)
-                instance._parent._on_change(instance._parent_field_name)
+                instance._parent._on_change([instance._parent_field_name])
 
 
 class BooleanDescriptor(FieldDescriptor):
@@ -120,7 +152,7 @@ class BinaryDescriptor(FieldDescriptor):
     default = None
 
     def __set__(self, instance, value):
-        assert isinstance(value, (basestring, buffer)) or value is None
+        assert isinstance(value, (bytes, bytearray)) or value is None
         super(BinaryDescriptor, self).__set__(instance, value)
 
 
@@ -193,7 +225,19 @@ class TimeDescriptor(FieldDescriptor):
         super(TimeDescriptor, self).__set__(instance, value)
 
 
+class TimeDeltaDescriptor(FieldDescriptor):
+    def __set__(self, instance, value):
+        assert isinstance(value, datetime.timedelta) or value is None
+        super(TimeDeltaDescriptor, self).__set__(instance, value)
+
+
 class DictDescriptor(FieldDescriptor):
+    def __get__(self, instance, owner):
+        value = super(DictDescriptor, self).__get__(instance, owner)
+        if value:
+            value = value.copy()
+        return value
+
     def __set__(self, instance, value):
         assert isinstance(value, dict) or value is None
         super(DictDescriptor, self).__set__(instance, value)
@@ -366,13 +410,13 @@ class MetaModelFactory(object):
         'integer': IntegerDescriptor,
         'biginteger': IntegerDescriptor,
         'float': FloatDescriptor,
-        'float_time': FloatDescriptor,
         'numeric': NumericDescriptor,
         'reference': ReferenceDescriptor,
         'date': DateDescriptor,
         'datetime': DateTimeDescriptor,
         'timestamp': DateTimeDescriptor,
         'time': TimeDescriptor,
+        'timedelta': TimeDeltaDescriptor,
         'dict': DictDescriptor,
         'many2one': Many2OneDescriptor,
         'one2many': One2ManyDescriptor,
@@ -468,7 +512,7 @@ class ModelList(list):
         'Signal change to parent'
         if self.parent:
             self.parent._changed.add(self.parent_field_name)
-            self.parent._on_change(self.parent_field_name)
+            self.parent._on_change([self.parent_field_name])
 
     def _get_context(self):
         from .pyson import PYSONDecoder
@@ -532,13 +576,15 @@ class ModelList(list):
         return res
     pop.__doc__ = list.pop.__doc__
 
-    def remove(self, record):
-        self.record_deleted.add(record)
+    def remove(self, record, _changed=True):
+        if record.id >= 0:
+            self.record_deleted.add(record)
         record._parent = None
         record._parent_field_name = ''
         record._parent_name = ''
         res = super(ModelList, self).remove(record)
-        self._changed()
+        if _changed:
+            self._changed()
         return res
     remove.__doc__ = list.remove.__doc__
 
@@ -660,6 +706,11 @@ class Model(object):
         'The unique ID'
         return self.__id
 
+    @id.setter
+    def id(self, value):
+        assert self.__id < 0
+        self.__id = int(value)
+
     @classmethod
     def find(cls, condition=None, offset=0, limit=None, order=None):
         'Return records matching condition'
@@ -669,46 +720,91 @@ class Model(object):
                 cls._config.context)
         return [cls(id) for id in ids]
 
-    def reload(self):
+    @dualmethod
+    def reload(cls, records):
         'Reload record'
-        self._values = {}
-        self._changed = set()
+        for record in records:
+            record._values = {}
+            record._changed = set()
 
-    def save(self):
-        'Save the record'
-        context = self._config.context
-        if self.id < 0:
-            values = self._get_values()
-            self.__id, = self._proxy.create([values], context)
-        else:
-            if not self._changed:
-                return
-            values = self._get_values(fields=self._changed)
-            context['_timestamp'] = self._get_timestamp()
-            self._proxy.write([self.id], values, context)
-        self.reload()
+    @dualmethod
+    def save(cls, records):
+        'Save records'
+        if not records:
+            return
+        proxy = records[0]._proxy
+        config = records[0]._config
+        context = config.context
+        create, write = [], []
+        for record in records:
+            assert proxy == record._proxy
+            assert config == record._config
+            if record.id < 0:
+                create.append(record)
+            elif record._changed:
+                write.append(record)
 
-    def delete(self):
-        'Delete the record'
-        if self.id > 0:
-            context = self._config.context
-            context['_timestamp'] = self._get_timestamp()
-            return self._proxy.delete([self.id], context)
-        self.reload()
-        return True
+        if create:
+            values = [r._get_values() for r in create]
+            ids = proxy.create(values, context)
+            for record, id_ in zip(create, ids):
+                record.id = id_
+        if write:
+            values = []
+            context['_timestamp'] = {}
+            for record in write:
+                values.append([record.id])
+                values.append(record._get_values(fields=record._changed))
+                context['_timestamp'].update(record._get_timestamp())
+            values.append(context)
+            proxy.write(*values)
+        for record in records:
+            record.reload()
 
-    @classmethod
+    @dualmethod
+    def delete(cls, records):
+        'Delete records'
+        if not records:
+            return
+        proxy = records[0]._proxy
+        config = records[0]._config
+        context = config.context
+        context['_timestamp'] = {}
+        delete = []
+        for record in records:
+            assert proxy == record._proxy
+            assert config == record._config
+            if record.id > 0:
+                context['_timestamp'].update(record._get_timestamp())
+                delete.append(record.id)
+        if delete:
+            proxy.delete(delete, context)
+        cls.reload(records)
+
+    @dualmethod
     def duplicate(cls, records, default=None):
         'Duplicate the record'
         ids = cls._proxy.copy([r.id for r in records], default,
             cls._config.context)
         return [cls(id) for id in ids]
 
-    def click(self, button):
+    @dualmethod
+    def click(cls, records, button, change=None):
         'Click on button'
-        self.save()
-        self.reload()  # Force reload because save doesn't always
-        return getattr(self._proxy, button)([self.id], self._config.context)
+        if not records:
+            return
+
+        proxy = records[0]._proxy
+        context = records[0]._config.context
+        if change is None:
+            cls.save(records)
+            cls.reload(records)  # Force reload because save doesn't always
+            return getattr(proxy, button)([r.id for r in records], context)
+        else:
+            record, = records
+            values = record._on_change_args(change)
+            changes = getattr(proxy, button)(values, context)
+            record._set_on_change(changes)
 
     def _get_values(self, fields=None):
         'Return dictionary values'
@@ -781,8 +877,7 @@ class Model(object):
             else:
                 self._values[field] = value
             fieldnames.append(field)
-        for field in sorted(fieldnames):
-            self._on_change(field)
+        self._on_change(sorted(fieldnames))
 
     def _get_eval(self):
         values = dict((x, getattr(self, '__%s_eval' % x))
@@ -845,8 +940,8 @@ class Model(object):
                             to_remove.append(record)
             for record in to_remove:
                 # remove without signal
-                list.remove(getattr(self, field), record)
-            if value and value.get('add') or value.get('update'):
+                getattr(self, field).remove(record, _changed=False)
+            if value and (value.get('add') or value.get('update')):
                 for index, vals in value.get('add', []):
                     relation = Model.get(self._fields[field]['relation'],
                             self._config)
@@ -875,58 +970,70 @@ class Model(object):
             self._values[field] = value
         self._changed.add(field)
 
-    def _on_change(self, name):
+    def _set_on_change(self, values):
+        later = {}
+        for field, value in values.iteritems():
+            if field not in self._fields:
+                continue
+            if self._fields[field]['type'] in ('one2many', 'many2many'):
+                later[field] = value
+                continue
+            self._on_change_set(field, value)
+        for field, value in later.iteritems():
+            self._on_change_set(field, value)
+
+    def _on_change(self, names):
         'Call on_change for field'
         # Import locally to not break installation
         from proteus.pyson import PYSONDecoder
-        definition = self._fields[name]
-        if definition.get('on_change'):
-            if isinstance(definition['on_change'], basestring):
-                definition['on_change'] = PYSONDecoder().decode(
-                        definition['on_change'])
-            args = self._on_change_args(definition['on_change'])
-            context = self._config.context
-            res = getattr(self._proxy, 'on_change_%s' % name)(args, context)
-            later = {}
-            for field, value in res.iteritems():
-                if field not in self._fields:
-                    continue
-                if self._fields[field]['type'] in ('one2many', 'many2many'):
-                    later[field] = value
-                    continue
-                self._on_change_set(field, value)
-            for field, value in later.iteritems():
-                self._on_change_set(field, value)
+
         values = {}
+        for name in names:
+            definition = self._fields[name]
+            on_change = definition.get('on_change')
+            if not on_change:
+                continue
+            if isinstance(on_change, basestring):
+                definition['on_change'] = on_change = PYSONDecoder().decode(
+                    on_change)
+            values.update(self._on_change_args(on_change))
+        if values:
+            context = self._config.context
+            changes = getattr(self._proxy, 'on_change')(values, names, context)
+            for change in changes:
+                self._set_on_change(change)
+
+        values = {}
+        fieldnames = set(names)
         to_change = set()
         later = set()
         for field, definition in self._fields.iteritems():
-            if not definition.get('on_change_with'):
+            on_change_with = definition.get('on_change_with')
+            if not on_change_with:
                 continue
-            if name not in definition['on_change_with']:
+            if not fieldnames & set(on_change_with):
                 continue
-            if field == name:
-                continue
-            if to_change & set(definition['on_change_with']):
+            if to_change & set(on_change_with):
                 later.add(field)
+                continue
             to_change.add(field)
-            values.update(self._on_change_args(definition['on_change_with']))
+            values.update(self._on_change_args(on_change_with))
         if to_change:
             context = self._config.context
-            result = getattr(self._proxy, 'on_change_with')(values,
+            changes = getattr(self._proxy, 'on_change_with')(values,
                 list(to_change), context)
-            for field, value in result.items():
-                self._on_change_set(field, value)
+            self._set_on_change(changes)
         for field in later:
-            definition = self._fields[field]
-            values = self._on_change_args(definition['on_change_with'])
+            on_change_with = self._fields[field]['on_change_with']
+            values = self._on_change_args(on_change_with)
             context = self._config.context
             result = getattr(self._proxy, 'on_change_with_%s' % field)(values,
                     context)
             self._on_change_set(field, result)
+
         if self._parent:
             self._parent._changed.add(self._parent_field_name)
-            self._parent._on_change(self._parent_field_name)
+            self._parent._on_change([self._parent_field_name])
 
 
 class Wizard(object):
@@ -997,6 +1104,9 @@ class Wizard(object):
 
         if self.state == self.end_state:
             self._proxy.delete(self.session_id, self._config.context)
+            if self.models:
+                for record in self.models:
+                    record.reload()
 
 
 class Report(object):
