@@ -1,5 +1,5 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 """
 Configuration functions for the proteus package for Tryton.
 """
@@ -20,7 +20,7 @@ def dump_decimal(self, value, write):
     self.dump_struct(value, write)
 
 
-def dump_buffer(self, value, write):
+def dump_bytes(self, value, write):
     self.write = write
     value = xmlrpclib.Binary(value)
     value.encode(self)
@@ -45,10 +45,19 @@ def dump_time(self, value, write):
         }
     self.dump_struct(value, write)
 
+
+def dump_timedelta(self, value, write):
+    value = {'__class__': 'timedelta',
+        'seconds': value.total_seconds(),
+        }
+    self.dump_struct(value, write)
+
 xmlrpclib.Marshaller.dispatch[Decimal] = dump_decimal
 xmlrpclib.Marshaller.dispatch[datetime.date] = dump_date
 xmlrpclib.Marshaller.dispatch[datetime.time] = dump_time
-xmlrpclib.Marshaller.dispatch[buffer] = dump_buffer
+xmlrpclib.Marshaller.dispatch[datetime.timedelta] = dump_timedelta
+xmlrpclib.Marshaller.dispatch[bytes] = dump_bytes
+xmlrpclib.Marshaller.dispatch[bytearray] = dump_bytes
 
 
 class XMLRPCDecoder(object):
@@ -70,6 +79,8 @@ XMLRPCDecoder.register('date',
 XMLRPCDecoder.register('time',
     lambda dct: datetime.time(dct['hour'], dct['minute'], dct['second'],
         dct['microsecond']))
+XMLRPCDecoder.register('timedelta',
+    lambda dct: datetime.timedelta(seconds=dct['seconds']))
 XMLRPCDecoder.register('Decimal', lambda dct: Decimal(dct['decimal']))
 
 
@@ -191,13 +202,14 @@ class TrytondProxy(object):
 class TrytondConfig(Config):
     'Configuration for trytond'
 
-    def __init__(self, database=None, user='admin', language='en_US',
-            password='', config_file=os.environ.get('TRYTOND_CONFIG')):
+    def __init__(self, database=None, user='admin', config_file=None):
         super(TrytondConfig, self).__init__()
         if not database:
             database = os.environ.get('TRYTOND_DATABASE_URI')
         else:
             os.environ['TRYTOND_DATABASE_URI'] = database
+        if not config_file:
+            config_file = os.environ.get('TRYTOND_CONFIG')
         from trytond.config import config
         config.update_etc(config_file)
         from trytond.pool import Pool
@@ -231,9 +243,9 @@ class TrytondConfig(Config):
     __init__.__doc__ = object.__init__.__doc__
 
     def __repr__(self):
-        return "proteus.config.TrytondConfig"\
-            "('%s', '%s', '%s', config_file=%s)"\
-            % (self.database, self._user, self.config_file)
+        return ("proteus.config.TrytondConfig"
+            "(%s, %s, config_file=%s)"
+            % (repr(self.database), repr(self._user), repr(self.config_file)))
     __repr__.__doc__ = object.__repr__.__doc__
 
     def __eq__(self, other):
@@ -261,11 +273,10 @@ class TrytondConfig(Config):
         return methods
 
 
-def set_trytond(database=None, user='admin', language='en_US', password='',
+def set_trytond(database=None, user='admin',
         config_file=None):
     'Set trytond package as backend'
-    _CONFIG.current = TrytondConfig(database, user, language=language,
-        password=password, config_file=config_file)
+    _CONFIG.current = TrytondConfig(database, user, config_file=config_file)
     return _CONFIG.current
 
 
@@ -286,17 +297,18 @@ class XmlrpcProxy(object):
 class XmlrpcConfig(Config):
     'Configuration for XML-RPC'
 
-    def __init__(self, url):
+    def __init__(self, url, **kwargs):
         super(XmlrpcConfig, self).__init__()
         self.url = url
-        self.server = xmlrpclib.ServerProxy(url, allow_none=1, use_datetime=1)
+        self.server = xmlrpclib.ServerProxy(
+            url, allow_none=1, use_datetime=1, **kwargs)
         # TODO add user
         self.user = None
         self._context = self.server.model.res.user.get_preferences(True, {})
     __init__.__doc__ = object.__init__.__doc__
 
     def __repr__(self):
-        return "proteus.config.XmlrpcConfig('%s')" % self.url
+        return "proteus.config.XmlrpcConfig(%s)" % repr(self.url)
     __repr__.__doc__ = object.__repr__.__doc__
 
     def __eq__(self, other):
@@ -320,9 +332,12 @@ class XmlrpcConfig(Config):
                 and '.' not in x[len(object_) + 1:]]
 
 
-def set_xmlrpc(url):
-    'Set XML-RPC as backend'
-    _CONFIG.current = XmlrpcConfig(url)
+def set_xmlrpc(url, **kwargs):
+    '''
+    Set XML-RPC as backend.
+    It pass the keyword arguments received to xmlrpclib.ServerProxy()
+    '''
+    _CONFIG.current = XmlrpcConfig(url, **kwargs)
     return _CONFIG.current
 
 
