@@ -1,9 +1,10 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 import os
 import sys
 import unittest
 
+
 def test_suite():
     suite = unittest.TestSuite()
     loader = unittest.TestLoader()
@@ -15,12 +16,13 @@ def test_suite():
             suite.addTests(loader.loadTestsFromModule(module))
     return suite
 
+
 def main():
     suite = test_suite()
     runner = unittest.TextTestRunner()
-    runner.run(suite)
+    return runner.run(suite)
 
 if __name__ == '__main__':
     sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.dirname(
         os.path.abspath(__file__)))))
-    main()
+    sys.exit(not main().wasSuccessful())
